# -*- coding: utf-8 -*-
from Products.CMFPlone import PloneMessageFactory as _
from Products.statusmessages.interfaces import IStatusMessage
from collective.easyform.interfaces import ISaveData
from plone import api
from plone.memoize.view import memoize
from Products.Five.browser import BrowserView


ATTENDEE_SAVE_DATA_ADAPTER_TITLE = 'Attendee Emails – DO NOT REMOVE'


class ViewAttendees(BrowserView):

    @memoize
    def attendee_emails(self):
        """return a list of the attendee email addresses"""
        all_email_addresses = []
        # read all registration forms in this conference
        brains = api.content.find(context=self.context, portal_type='EasyForm')
        for brain in brains:
            form = brain.getObject()
            # get all the form's save data adapters
            all_save_data_adapters = form.restrictedTraverse("@@get_save_data_adapters")
            results = all_save_data_adapters()
            for saver in results:
                # must have the ID 'attendee_emails___do_not_remove'
                if saver.title == ATTENDEE_SAVE_DATA_ADAPTER_TITLE:
                    save_data_adapter = ISaveData(saver)
                    data = save_data_adapter.getSavedFormInput()
                    # relying on the email address field to have the ID 'replyto'
                    if len(data) > 0 and 'replyto' in data[0].keys():
                        email_addresses = [item['replyto'] for item in data if item['approved']]
                        all_email_addresses.extend(email_addresses)
        unique_email_addresses_set = set(all_email_addresses)
        return list(unique_email_addresses_set)

    @memoize
    def bcc_email_link(self):
        """return a mailto URL"""
        email_addresses = self.attendee_emails()
        email_addresses_string = ",".join([email for email in email_addresses])
        return f"mailto:?bcc={email_addresses_string}"

    @memoize
    def forms_with_attendee_save_data_adapters(self):
        """return only the attendee save data adapters in this conference"""
        forms = []
        brains = api.content.find(context=self.context, portal_type='EasyForm')
        for brain in brains:
            form = brain.getObject()
            # get all the form's save data adapters
            all_save_data_adapters = form.restrictedTraverse("@@get_save_data_adapters")
            results = all_save_data_adapters()
            for saver in results:
                # must have the ID 'attendee_emails___do_not_remove'
                if saver.title == ATTENDEE_SAVE_DATA_ADAPTER_TITLE:
                    forms.append(form)
        return forms

    @memoize
    def attendee_group_name(self):
        conference = self.context
        return f"{conference.id}-attendees"


class UpdateAttendeesGroupView(BrowserView):

    def __call__(self, *args, **kwargs):
        # get the email addresses of attendees
        attendees_view = api.content.get_view('view-attendees', self.context, self.request)
        attendee_emails = attendees_view.attendee_emails()
        attendee_emails_set = set(attendee_emails)
        conference = self.context
        attendees_group_name = f"{conference.id}-attendees"
        # get the users
        attendees_users = []
        for attendee in attendee_emails:
            user = api.user.get(username=attendee)
            if user:
                attendees_users.append(user)
        # look up who is in this conference's attendees group
        group = api.group.get(groupname=attendees_group_name)
        if group:
            member_emails = []
            members = api.user.get_users(groupname=attendees_group_name)
            for member in members:
                email = member.getProperty('email')
                if len(email) > 0:
                    member_emails.append(email)
            member_emails_set = set(member_emails)
        # do a set diff
        attendees_not_yet_in_group = attendee_emails_set - member_emails_set
        attendees_to_remove_from_group = member_emails_set - attendee_emails_set
        if attendees_not_yet_in_group or attendees_to_remove_from_group:
            # add needed attendees to group
            for attendee in attendees_not_yet_in_group:
                user = api.user.get(username=attendee)
                if not user:
                    api.user.create(username=attendee, email=attendee)
                api.group.add_user(groupname=attendees_group_name, username=attendee)
            # remove unneeded attendees from group
            for attendee in attendees_to_remove_from_group:
                api.group.remove_user(groupname=attendees_group_name, username=attendee)
            IStatusMessage(self.request).add(
                _(u'The attendees group has been updated.'))
            self.request.response.redirect(self.request.HTTP_REFERER)
        else:
            IStatusMessage(self.request).add(
                _(u'No change was made to the attendees group.'))
            self.request.response.redirect(self.request.HTTP_REFERER)
